@@ -6,6 +6,7 @@ import socket
 import sys
 import threading
 import time
+from functools import lru_cache
 from typing import Dict, Optional, Union
 from urllib.parse import urlparse
 
@@ -24,6 +25,7 @@ from localstack.constants import (
     APPLICATION_AMZ_JSON_1_0,
     APPLICATION_AMZ_JSON_1_1,
     APPLICATION_X_WWW_FORM_URLENCODED,
+    AWS_REGION_US_EAST_1,
     ENV_DEV,
     INTERNAL_AWS_ACCESS_KEY_ID,
     LOCALHOST,
@@ -35,18 +37,12 @@ from localstack.constants import (
     TEST_AWS_ACCOUNT_ID,
     TEST_AWS_SECRET_ACCESS_KEY,
 )
-from localstack.utils.aws import templating
 from localstack.utils.aws.aws_models import KinesisStream
-from localstack.utils.common import (
-    get_service_protocol,
-    is_string,
-    is_string_or_bytes,
-    make_http_request,
-    retry,
-    run_safe,
-    to_str,
-)
-from localstack.utils.generic import dict_utils
+from localstack.utils.collections import pick_attributes
+from localstack.utils.functions import run_safe
+from localstack.utils.http import make_http_request
+from localstack.utils.strings import is_string, is_string_or_bytes, to_str
+from localstack.utils.sync import retry
 
 # AWS environment variable names
 ENV_ACCESS_KEY = "AWS_ACCESS_KEY_ID"
@@ -84,6 +80,15 @@ CACHE_S3_HOSTNAME_DNS_STATUS = None
 BOTO_CLIENT_CREATE_LOCK = threading.RLock()
 
 
+@lru_cache()
+def get_valid_regions():
+    valid_regions = set()
+    for partition in set(boto3.Session().get_available_partitions()):
+        for region in boto3.Session().get_available_regions("sns", partition):
+            valid_regions.add(region)
+    return valid_regions
+
+
 class Environment(object):
     def __init__(self, region=None, prefix=None):
         # target is the runtime environment to use, e.g.,
@@ -239,7 +244,7 @@ def set_internal_auth(headers):
 def get_local_service_url(service_name_or_port: Union[str, int]) -> str:
     """Return the local service URL for the given service name or port."""
     if isinstance(service_name_or_port, int):
-        return f"{get_service_protocol()}://{LOCALHOST}:{service_name_or_port}"
+        return f"{config.get_protocol()}://{LOCALHOST}:{service_name_or_port}"
     service_name = service_name_or_port
     if service_name == "s3api":
         service_name = "s3"
@@ -263,6 +268,27 @@ def connect_to_resource(
     )
 
 
+def connect_to_resource_external(
+    service_name,
+    env=None,
+    region_name=None,
+    endpoint_url=None,
+    config: botocore.config.Config = None,
+    **kwargs,
+):
+    """
+    Generic method to obtain an AWS service resource using boto3, based on environment, region, or custom endpoint_url.
+    """
+    return create_external_boto_client(
+        service_name,
+        client=False,
+        env=env,
+        region_name=region_name,
+        endpoint_url=endpoint_url,
+        config=config,
+    )
+
+
 def connect_to_service(
     service_name,
     client=True,
@@ -374,11 +400,6 @@ def get_s3_hostname():
     return LOCALHOST
 
 
-# TODO remove from here in the future
-def render_velocity_template(*args, **kwargs):
-    return templating.render_velocity_template(*args, **kwargs)
-
-
 def generate_presigned_url(*args, **kwargs):
     endpoint_url = kwargs.pop("endpoint_url", None)
     s3_client = connect_to_service(
@@ -404,8 +425,8 @@ def check_valid_region(headers):
     # See https://docs.aws.amazon.com/AmazonS3/latest/API/sigv4-auth-using-authorization-header.html
     parts = replaced.split("/")
     region = parts[2]
-    if region not in config.VALID_REGIONS:
-        raise Exception('Invalid region specified in "Authorization" header: "%s"' % region)
+    if region not in get_valid_regions():
+        raise Exception(f'Invalid region specified in "Authorization" header: "{region}"')
 
 
 def set_default_region_in_headers(headers, service=None, region=None):
@@ -588,12 +609,14 @@ def get_iam_role(resource, env=None):
     return "role-%s" % resource
 
 
-# TODO: remove this (can't statically define secret ARN because it includes a random suffix)
-def secretsmanager_secret_arn(secret_id, account_id=None, region_name=None):
+def secretsmanager_secret_arn(secret_id, account_id=None, region_name=None, random_suffix=None):
     if ":" in (secret_id or ""):
         return secret_id
     pattern = "arn:aws:secretsmanager:%s:%s:secret:%s"
-    return _resource_arn(secret_id, pattern, account_id=account_id, region_name=region_name)
+    arn = _resource_arn(secret_id, pattern, account_id=account_id, region_name=region_name)
+    if random_suffix:
+        arn += f"-{random_suffix}"
+    return arn
 
 
 def cloudformation_stack_arn(stack_name, stack_id=None, account_id=None, region_name=None):
@@ -702,7 +725,7 @@ def fix_arn(arn):
     resource names to ARNs, replacing incorrect regions, account IDs, etc."""
     if arn.startswith("arn:aws:lambda"):
         parts = arn.split(":")
-        region = parts[3] if parts[3] in config.VALID_REGIONS else get_region()
+        region = parts[3] if parts[3] in get_valid_regions() else get_region()
         return lambda_function_arn(lambda_function_name(arn), region_name=region)
     LOG.warning("Unable to fix/canonicalize ARN: %s", arn)
     return arn
@@ -769,15 +792,26 @@ def _resource_arn(name: str, pattern: str, account_id: str = None, region_name:
 
 
 def get_events_target_attributes(target):
-    return dict_utils.pick_attributes(target, EVENT_TARGET_PARAMETERS)
+    return pick_attributes(target, EVENT_TARGET_PARAMETERS)
 
 
-def get_or_create_bucket(bucket_name, s3_client=None):
+def get_or_create_bucket(bucket_name: str, s3_client=None):
     s3_client = s3_client or connect_to_service("s3")
     try:
         return s3_client.head_bucket(Bucket=bucket_name)
     except Exception:
-        return s3_client.create_bucket(Bucket=bucket_name)
+        return create_s3_bucket(bucket_name, s3_client=s3_client)
+
+
+def create_s3_bucket(bucket_name: str, s3_client=None):
+    """Creates a bucket in the region that is associated with the current request
+    context, or with the given boto3 S3 client, if specified."""
+    s3_client = s3_client or connect_to_service("s3")
+    region = s3_client.meta.region_name
+    kwargs = {}
+    if region != AWS_REGION_US_EAST_1:
+        kwargs = {"CreateBucketConfiguration": {"LocationConstraint": region}}
+    return s3_client.create_bucket(Bucket=bucket_name, **kwargs)
 
 
 def create_sqs_queue(queue_name, env=None):
@@ -844,10 +878,9 @@ def mock_aws_request_headers(service="dynamodb", region_name=None, access_key=No
         "X-Amz-Date": "20160623T103251Z",
         "Authorization": (
             "AWS4-HMAC-SHA256 "
-            + "Credential=%s/20160623/%s/%s/aws4_request, "
+            + f"Credential={access_key}/20160623/{region_name}/{service}/aws4_request, "
             + "SignedHeaders=content-type;host;x-amz-date;x-amz-target, Signature=1234"
-        )
-        % (access_key, region_name, service),
+        ),
     }
     return headers
 
